/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/
//-----------------------------------------------------------------------------
// V/f prog pour MRVL / MASL
// dsPIC 30F3010
// L. BAGHLI 05/01/2006
// version actuelle : actionneur  plots AUTOPILOT 15/04/2006 (channel 4 RB5)
// en mode MRV : commutation par action sur OVDCON, valeur de la tension par PDCx
//-----------------------------------------------------------------------------
#include "p30F3010.h"
// 3 modes :
// MRV			:		(actionneur linaire) alimentation par crnaux squentiels
// MRV_autopiotage	:		letcture de valeur du capteur IR sur RB5 (ADC) puis commutation si > seuil
// MAS			:		excitation sinusoidale en V et f independants

#include "sinus.h"

//Configuration bits
_FOSC(CSW_FSCM_OFF & XT_PLL8);
_FWDT(WDT_OFF);
_FBORPOR(PBOR_OFF & BORV_27 & PWRT_16 & MCLR_EN);
_FGS(CODE_PROT_OFF);
//-----------------------------------------------------------------------------
//Program Specific Constants
#define FCY 8000000						//Instruction cycle rate (Osc x PLL / 4)
#define FPWM 10000						//PWM freq
#define MILLISEC FCY/8000			// 1 mSec delay constant

#define StartPrg		PORTBbits.RB1
#define TogglesAlim	PORTDbits.RD0
#define EnableL6234	PORTDbits.RD1
#define RunningLED	PORTBbits.RB0
#define InterruptLED	PORTBbits.RB4
//#define BugLED			PORTBbits.RB5		// no more used in autopilotage car entree analogique ! 
#define BugLED			PORTEbits.RE0		// la pin PWM1 low ! 


void setup_ports(void);
void InitADC10(void);
void DelayNmSec(unsigned int N);
void InitMCPWM(void);
void InitUART(void);
void SendMsg(void);

struct {
			unsigned RunMotor : 	1;
			unsigned Sendfs		:		1;
			unsigned CheckRX	:		1;
			unsigned SendTX		:		1;
			unsigned MAS			:		1;
			unsigned MRV			:		1;
			unsigned MRV_autopilotage	:		1;
			unsigned ModeFrontRev	:		1;
			unsigned FrontRev	:		1;
			unsigned CaptIRState	:		1;
			unsigned OldCaptIRState	:		1;
			unsigned OldTogglesAlimState	:		1;
			unsigned unused 	:		4;
		}	Flags;
unsigned char *TXPtr;
unsigned char *RXPtr;

int k_V_f;		// rapport V/f (lu sur l'ADC AN2/RB2)
int	fs;				// fs freq de ref (lu sur l'ADC AN3/RB3)
int	CaptIR;		// valeur en tension du signal du capteur IR pour l'autopilotage (lu sur l'ADC AN5/RB5)
unsigned int thetas;
int thetasinc;
int sinthetas, costhetas;
int tmp1, tmp2;
int Vds, Valphas, Vbetas, Vas, Vbs, Vcs;
#define Ktheta	262			// 65536/250
#define SQRT3_2 3547		// sqrt(3)/2 en 4.12
#define ByEs2 800		// en 4.12

int Sequence;
int SeqMaxDelay;
int SeqDelayCounter;
// le nbre de pas ds un sens avant le changement de direction
int FrontRearCounter;
#define FrontRearCounterMax 50

#define CR	0x0D
#define LF 	0x0A
#define BAUD 19200
#define OFFSET 5				// offset in OutData : position de la val de fs
unsigned char InData[] = {"000000"};
unsigned char OutData[] = {"fs = 00.0 Hz\r\n"};
int SeqComm;
#define SeqCommMax 3000

//-----------------------------------------------------------------------------
//	Initialise variables
//-----------------------------------------------------------------------------
void InitVar(void)
{
	if (Flags.MAS)
		{
		thetas=0;
		Vas=0;
		Vbs=0;
		Vcs=0;
		Valphas=0;
		Vbetas=0;
		}
	else
		{
		Sequence=0;				// init des variables du deroulement de la seq d'alim des enroulements
		SeqMaxDelay=10;
		SeqDelayCounter=SeqMaxDelay;
		Flags.OldCaptIRState=0;
		Flags.OldTogglesAlimState=0;
		}
	Flags.ModeFrontRev=0;
	Flags.FrontRev=1;
}
//-----------------------------------------------------------------------------
//	Setup ports
//-----------------------------------------------------------------------------
void setup_ports(void)
{
	// Clear All Ports Prior to defining I/O
//	ADPCFG = 0xFFF3;				// all PORTB = Digital(1) ;RB2 et RB3 = analog(0)  ie 0011
	ADPCFG = 0xFFD3;				// all PORTB = Digital(1) ;RB2, RB3 et RB5 = analog(0)  ie F:F:1101:0011
	PORTB=0;	//Initialize LED pin data to off state
	PORTC=0;
	PORTD=0;	//disable L6234 RD1=Off
	PORTE=0;
	// Now set pin direction registers
//	TRISB = 0xFFCE;	// RB0 LED1 out, RB1 BP2 in, RB2 adc in, RB3 adc in, RB4 LED2 out, RB5 LED3 out
	TRISB = 0xFFEE;	// RB0 LED1 out, RB1 BP2 in, RB2 adc in, RB3 adc in, RB4 LED2 out, RB5in (adc one !!!)
	TRISC = 0xDFFF;			// C13  TX out, C14 RX in, the rest is input / Unused
	TRISD =	0xFFFD; //Set pin as : RD0 BP3 input=1, RD1 output=0 enable L6234, NO debug allowed on EMUC2/EMUD2
	TRISE = 0x0000;		// RE5-0 are the PWMs -> outputs sauf les low : RE4, RE2, RE0, no FLTA use
}
//-----------------------------------------------------------------------------
//	Setup the ADC registers for :
//	2 channels conversion (in this case RB2/AN2)
//	PWM trigger starts conversion
//	Pot K_Vf is connected to CH0 and RB2
//	Pot f		 is connected to CH1 and RB3
//	AD interrupt is set and buffer is read in the interrupt 
//-----------------------------------------------------------------------------
void InitADC10(void)
{
 ADCON1 = 0x006F;				// PWM starts conversion, 4 ch simultans, ADC Off for configuring
	if (Flags.MAS)
		ADCON2 = 0x020C;				// 4 simulataneous sample 4 channels, ADC INTerrupt  chaque 4 EOC=400 us car a rempli deja 16 buffer 
	else
		 ADCON2 = 0x0200;				// simulataneous sample 4 channels, ADC INTerrupt  chaque EOC=100 us
// ADCON2 = 0x023C;				// simulataneous sample 4 channels	, ADC interrupt every 16 EOC
 ADCHS	= 0x0022;				// Connect RB2/AN2 as CH0 = pot K_V_f  AN3/RB3 to CH1 = pot Fs, AN5/RB5 to CH3 = lecture valeur du capteur IR
 ADCON3 = 0x0080;				// Tad = internal RC (4uS)
 IFS0bits.ADIF = 0;			// Adc int flag Off
 IEC0bits.ADIE = 1;			// Adc int On
 ADCON1bits.ADON = 1;			// turn ADC ON
}

//-----------------------------------------------------------------------------
//  InitMCPWM, intializes the Motor PWM as follows:
//  FPWM = 10 khz at 8Mips
//  Independant PWMs
//  Control outputs using OVDCON
//  Init Duty Cycle with a value = 100
//  Set ADC to be triggered by PWM special trigger
//-----------------------------------------------------------------------------
void InitMCPWM(void)
{
	PTPER = FCY/FPWM - 1;		// set the pwm period register
	PWMCON1 = 0x0700;			// disable PWMs
	OVDCON = 0x0000;			// allow control using OVD register
	PDC1 = 800;					// init PWM DC 1, 2 and 3 to 800= 0 tension (50% duty cycle)
	PDC2 = 800;
	PDC3 = 800;
	SEVTCMP = PTPER;		// set ADC to trigeer at ...
//	PWMCON2 = 0x0F00;		// 16 PWM values
	PWMCON2 = 0x0000;		// 1 PWM values
	PTCON = 0x8000;			// start PWM however output ...
							// is enabled by OVDCON which is OFF
}

//---------------------------------------------------------------------
// Below are the interrupt vectors for the serial receive and transmit
//---------------------------------------------------------------------
void __attribute__((__interrupt__)) _U1TXInterrupt(void)
{
	IFS0bits.U1TXIF = 0;	// clear interrupt flag
}
//---------------------------------------------------------------------
void __attribute__((__interrupt__)) _U1RXInterrupt(void)
{
	IFS0bits.U1RXIF = 0;	// clear interrupt flag
	*RXPtr = U1RXREG;
	if (*RXPtr == CR)
		{Flags.CheckRX = 1;RXPtr = &InData[0];}
	else *RXPtr++;
}

//---------------------------------------------------------------------
//	The ADC interrupt reads the demand pot value. 
// tt est synchrone %  cette int de 100 us
//---------------------------------------------------------------------
void __attribute__((__interrupt__)) _ADCInterrupt (void)
{
	InterruptLED=1;	
	IFS0bits.ADIF = 0;	

	if (Flags.MAS)
		{
		k_V_f=ADCBUF0<<2;			// 4.12 pu 4096=1.0=6V
		fs=ADCBUF1<<2;				// 4.12 pu 4096=1.0=20 Hz
		thetasinc=__builtin_mulss(Ktheta,fs)>>12;
		if (Flags.FrontRev)	thetas+=thetasinc;
				else						thetas-=thetasinc;
		sinthetas=SinusTable[thetas>>8];
		costhetas=SinusTable[((thetas>>8)+64) & 255];
		Vds=k_V_f;
		Valphas=__builtin_mulss(Vds,costhetas)>>12;
		Vbetas =__builtin_mulss(Vds,sinthetas)>>12;
		Vas=Valphas;
		tmp1=-(Valphas/2);
		tmp2=(__builtin_mulss(SQRT3_2,Vbetas))>>12;
		Vbs=tmp1+tmp2;
		Vcs=tmp1-tmp2;
		tmp1=(__builtin_mulss(ByEs2,Vas))>>12;
		PDC1=800+tmp1;
		tmp1=+(__builtin_mulss(ByEs2,Vbs))>>12;
		PDC2=800+tmp1;
		tmp1=(__builtin_mulss(ByEs2,Vcs))>>12;
		PDC3=800+tmp1;
		}
	else
		{
		// actionneur  plot (MRV linaire 6/4)
		k_V_f	=ADCBUF0<<1;
		fs		=ADCBUF1;
		fs=fs*(fs>>4);
		CaptIR		=ADCBUF3;	// capteur pour l'autopilotage : > 3V trait noir donc commutation
		if (CaptIR>512)		Flags.CaptIRState=1;
							else		Flags.CaptIRState=0;
		PDC1=k_V_f;	// meme tension pour les 3 phases, mais chacun son tour
		PDC2=k_V_f;
		PDC3=k_V_f;
		if (Flags.MRV_autopilotage)
						{
						if (Flags.CaptIRState!=Flags.OldCaptIRState && Flags.CaptIRState)	// change d'etat
																									{
																									if (++Sequence>=3)	Sequence=0;
																									if (Sequence==0)	OVDCON=0x0300;	// phase A active only 
																									if (Sequence==1)	OVDCON=0x0C00;	// phase B active only 
																									if (Sequence==2)	OVDCON=0x3000;	// phase C active only 
																									}
						Flags.OldCaptIRState=Flags.CaptIRState;
						}
			else	{
						SeqMaxDelay=fs+1;		// la vitesse de deroulement des sequences, plus fs est grande plus c'est lent !
						// ds le mode MRV, si on appuie sur (TogglesAlim ie PB3)
						// alors on toggles du mode demo (inversion au bout de 20 pas), vers le mode normal (vitesse unidirectionnelle)
						if (TogglesAlim!=Flags.OldTogglesAlimState && TogglesAlim)	// change d'etat vers	 appuy
																									{
																									if (Flags.ModeFrontRev)		{Flags.ModeFrontRev=0; BugLED=0;}		// allume la LED 
																																else				{
																																						Flags.ModeFrontRev=1; BugLED=1;
																																						FrontRearCounter=FrontRearCounterMax;
																																						}
																									}
						Flags.OldTogglesAlimState=TogglesAlim;
						if (!--SeqDelayCounter)
							{
							SeqDelayCounter=SeqMaxDelay;
							if (Flags.FrontRev)	{if (++Sequence>=3)	Sequence=0;
																	}
												else			{if (--Sequence<0)	Sequence=2;
																	}
							if	(Flags.ModeFrontRev)	if (!--FrontRearCounter)	{
																														FrontRearCounter=FrontRearCounterMax;
																														if (Flags.FrontRev)	Flags.FrontRev=0;			// change de direction : recule
																																	else					Flags.FrontRev=1;			// change de direction : avance
																														}
							if (Sequence==0)	OVDCON=0x0300;	// phase A active only 
							if (Sequence==1)	OVDCON=0x0C00;	// phase B active only 
							if (Sequence==2)	OVDCON=0x3000;	// phase C active only 
							}	
						}
	}
	// communication dsPIC -> PC
	if (!--SeqComm)		{
										SeqComm=SeqCommMax;
										Flags.Sendfs=1;
										}
	
	InterruptLED=0;	
}

//------------------------------------------------------------------------
//	Transmission over serial
void InitUART(void)
{
// Initialize the UART1 for BAUD = 19,200  
	U1MODE = 0x8400;	// enable + alternate pins
	U1STA = 0x0000;
	U1BRG = ((FCY/16)/BAUD) - 1;	// set baud to 19200
	IEC0bits.U1RXIE = 1;			// enable RX interrupt
	RXPtr = &InData[0];		// point to first char in receive buffer
	Flags.CheckRX = 0;			// clear rx and tx flags
	Flags.SendTX = 0;

	U1STAbits.UTXEN = 1;           // Initiate transmission
	SeqComm=SeqCommMax;
}

//------------------------------------------------------------------------
// Sendfs sends the fs information on the uart at 19200 baud
void Sendfs()
{
unsigned int k;
unsigned char c;
	
	k = fs/2.048;		//50.0Hz -> 500 -> 5V -> 1024
	c = k/100;
	if (c > 0)
		k = k - c*100;
	OutData[OFFSET] = (c + 0x30);
	c = k/10;
	if (c > 0)
		k = k - c*10;
	OutData[OFFSET+1] = (c + 0x30);
	// evite le point dcimal
	OutData[OFFSET+3] = (char)(k + 0x30);
	TXPtr = &OutData[0];
	SendMsg();
}

void SendMsg(void)
{
while (*TXPtr)
	{
	while (U1STAbits.UTXBF);
	U1TXREG = *TXPtr++;
	}
}

//-----------------------------------------------------------------------------
//Main routine
int main(void)
{
	setup_ports();
//	BugLED = 0; //Turn LED pb off
//	InterruptLED = 0; //Turn LED pb off
//  ^^^^ dja fait ds le setup port, mais introduit un temps d'attente

	// check user choice : MRV, MRVmanuelle ou MAS 
	if (StartPrg) {
								Flags.MAS=1;		//alors mode MAS
								Flags.MRV=0;
								Flags.MRV_autopilotage=0;
								DelayNmSec(500);		// wait 0.5 s
								BugLED = 1;
								if (StartPrg)	// still pressed  alors mode MRV_autopilotage
										{
										Flags.MAS=0;
										Flags.MRV=0;
										Flags.MRV_autopilotage=1;
										}
								while (StartPrg)	DelayNmSec(10);		// wait 'till key is released
								}
			else			{
								Flags.MAS=0;
								Flags.MRV=1;
								Flags.MRV_autopilotage=0;
								}
	BugLED = 0;
	while (StartPrg)	DelayNmSec(10);		// wait 'till key is released

	InitVar();
	InitMCPWM();
	InitADC10();
	InitUART();
	while(1)
	{	
		while (!StartPrg);									// wait for start key hit
		while (StartPrg)	DelayNmSec(10);		// wait 'till key is released
		RunningLED=1;						// LED run = On
		if (Flags.MAS)
				OVDCON = 0xFFFF;	// Cmde moteur triphas, no effect of OVDCON
		else
				{
				if (Sequence==0)	OVDCON=0x0300;	// phase A active only 
				if (Sequence==1)	OVDCON=0x0C00;	// phase B active only 
				if (Sequence==2)	OVDCON=0x3000;	// phase C active only 
				}
		PWMCON1= 0x0770;					// enable PWM outputs (mais pas les LOW)
		EnableL6234=1;						// enable L6234
		Flags.RunMotor = 1;					// set running flag
		DelayNmSec(100);
		while (Flags.RunMotor)				// while motor is running
			{
			if (StartPrg)							// if S2 is pressed, stop
				{
				EnableL6234=0;						// desable L6234
				PWMCON1 = 0x0700;			// disable PWM outputs
	  		OVDCON = 0x0000;			// overide PWM low.
				Flags.RunMotor = 0;			// reset run flag
				RunningLED=0;						// LED run = Off
				while (StartPrg)	DelayNmSec(10);		// wait 'till key is released
				}

			if (Flags.Sendfs)
				{
				Sendfs();		// send present fs serially
				Flags.Sendfs = 0;		// clear flag 
				}
			}
//		BugLED = 1; //pb ???
	}	// end of while (1)
}	// end of main

//=============================================================================
//Error traps
//-----------------------------------------------------------------------------
//Oscillator Fail Error trap routine
void _ISR _OscillatorFail(void)
{
//	LATDbits.LATD1 = 1; //Turn LED pb on
	BugLED=1;
	while(1); //Wait forever
}
//-----------------------------------------------------------------------------
//Address Error trap routine
void _ISR _AddressError(void)
{
	BugLED=1;
	while(1); //Wait forever
}
//-----------------------------------------------------------------------------
//Stack Error trap routine
void _ISR _StackError(void)
{
	BugLED=1;
	while(1); //Wait forever
}
//-----------------------------------------------------------------------------
//Math (Arithmetic) Error trap routine
void _ISR _MathError(void)
{
	BugLED=1;
	while(1); //Wait forever
}
//---------------------------------------------------------------------
// This is a generic 1ms delay routine to give a 1mS to 65.5 Seconds delay
// For N = 1 the delay is 1 mS, for N = 65535 the delay is 65,535 mS. 
// Note that FCY is used in the computation.  Please make the necessary
// Changes(PLLx4 or PLLx8 etc) to compute the right FCY as in the define
// statement above.
//---------------------------------------------------------------------
void DelayNmSec(unsigned int N)
{
unsigned int j;
while(N--)
 	for(j=0;j < MILLISEC;j++);
}
//---------------------------------------------------------------------
